local pcall = pcall
local unpack = unpack
local error = error
local tostring = tostring
local type = type
local string = string

local oop = require "loop.base"

---
-- Classe utilizada para assegurar certas condies esperadas pelo programador
--  dos testes.
module("latt.Check", oop.class)

local function assertBoolean(condition)
  if type(condition) ~= "boolean" then
    error("The condition is not a boolean value.", 3)
  end
end

---
-- Assegura que uma determinada funo est lanando um erro.
-- Caso isto no acontea, um erro  lanado indicando que a funo foi
-- executada com sucesso.
--
-- @param f A funo
-- @param ... Os parmetros que sero passados para a funo
--
function assertError(f, ...)
  local success = pcall(f, unpack(arg))
  if success then
    error("Function shouldn't run successfully", 2)
  end
end

---
-- Asegura que dois valores so iguais.
-- Caso isto no acontea, um erro  lanado indicando que os dois valores so
-- diferentes.
--
-- @param expected O primeiro valor
-- @param actual O segundo valor
-- @param userMessage Uma mensagem que ser anexada ao erro caso os valores no
-- sejam iguais.
--
function assertEquals(expected, actual, userMessage)
  if expected ~= actual then
    local errorMessage = "The expected value ["..tostring(expected)..
        "] must be equal the actual value ["..tostring(actual).."]"
    if userMessage then
      errorMessage = errorMessage..": "..userMessage
    end
    error(errorMessage, 2)
  end
end

---
-- Asegura que dois valores so diferentes.
-- Caso isto no acontea, um erro  lanado indicando que os dois valores so
-- iguais.
--
-- @param expected O primeiro valor
-- @param actual O segundo valor
-- @param userMessage Uma mensagem que ser anexada ao erro caso os valores
-- sejam iguais.
--
function assertNotEquals(expected, actual, userMessage)
  if expected == actual then
    local errorMessage = "The expected value ["..tostring(expected)..
        "] shouldn't be equal the actual value ["..tostring(actual).."]"
    if userMessage then
      errorMessage = errorMessage..": "..userMessage
    end
    error(errorMessage, 2)
  end
end

---
-- Asegura que uma determinada condio  verdadeira.
-- Caso isto no acontea, um erro  lanado indicando que a condio  falsa.
-- Caso a condio no seja uma varivel do tipo boolean, um erro  lanado.
--
-- @param condition A condio (tipo boolean)
-- @param userMessage Uma mensagem que ser anexada ao erro caso a condio
-- seja falsa.
--
function assertTrue(condition, userMessage)
  assertBoolean(condition)
  if condition == false then
    local errorMessage = "The condition must be true"
    if userMessage then
      errorMessage = errorMessage..": "..userMessage
    end
    error(errorMessage, 2)
  end
end

---
-- Asegura que uma determinada condio  falsa.
-- Caso isto no acontea, um erro  lanado indicando que a condio 
-- verdadeira.
-- Caso a condio no seja uma varivel do tipo boolean, um erro  lanado.
--
-- @param condition A condio (tipo boolean)
-- @param userMessage Uma mensagem que ser anexada ao erro caso a condio
-- seja verdadeira.
--
function assertFalse(condition, userMessage)
  assertBoolean(condition)
  if condition == true then
    local errorMessage = "The condition must be false"
    if userMessage then
      errorMessage = errorMessage..": "..userMessage
    end
    error(errorMessage, 2)
  end
end

---
-- Asegura que uma determinada varivel  nula.
-- Caso isto no acontea, um erro  lanado indicando que a varivel no 
-- nula.
--
-- @param variable A varivel
-- @param userMessage Uma mensagem que ser anexada ao erro caso a varivel no
-- seja nula
--
function assertNil(variable, userMessage)
  if variable ~= nil then
    local errorMessage = "The variable must be nil, but the cctual value is ["..
        tostring(variable).."]"
    if userMessage then
      errorMessage = errorMessage..": "..userMessage
    end
    error(errorMessage, 2)
  end
end

---
-- Asegura que uma determinada varivel no  nula.
-- Caso isto no acontea, um erro  lanado indicando que a varivel  nula.
--
-- @param variable A varivel
-- @param userMessage Uma mensagem que ser anexada ao erro caso a varivel
-- seja nula
--
function assertNotNil(variable, userMessage)
  if variable == nil then
    local errorMessage = "The variable shouldn't be nil"
    if userMessage then
      errorMessage = errorMessage..": "..userMessage
    end
    error(errorMessage, 2)
  end
end
